----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    14:33:10 04/07/2010 
-- Design Name: 
-- Module Name:    AFTITERPHASE - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity AFTITERPHASE is
    Port ( rst : in  STD_LOGIC;
           clk : in  STD_LOGIC;
           valid_in : in  STD_LOGIC;
           pLUTaddr : in  STD_LOGIC_VECTOR (7 downto 0);
           mLUTaddr : in  STD_LOGIC_VECTOR (11 downto 0);
           iter_val : in  STD_LOGIC_VECTOR (31 downto 0);
           valid_out : out  STD_LOGIC;
           value_out : out  STD_LOGIC_VECTOR (31 downto 0));
end AFTITERPHASE;

architecture Behavioral of AFTITERPHASE is

component sp_fp_add is
  port (
    rdy : out STD_LOGIC; 
    operation_nd : in STD_LOGIC := 'X'; 
    clk : in STD_LOGIC := 'X'; 
    a : in STD_LOGIC_VECTOR ( 31 downto 0 ); 
    b : in STD_LOGIC_VECTOR ( 31 downto 0 ); 
    result : out STD_LOGIC_VECTOR ( 31 downto 0 ) 
  );
end component;

signal add_vld_o: std_logic;
constant const_val: std_logic_Vector(31 downto 0):="00111111100000000000000000000000";
signal add_res,mult1_res,mult1_res_reg: std_logic_Vector(31 downto 0);

component EAUmLUT is
    Port ( clk : in  STD_LOGIC;
           addr : in  STD_LOGIC_VECTOR (11 downto 0);
           data : out  STD_LOGIC_VECTOR (31 downto 0));
end component;

signal mLUT_data, mLUT_data_sftd: std_logic_vector(31 downto 0);

component shift_reg_32b_9c is
  port (
    clk : in STD_LOGIC := 'X'; 
    d : in STD_LOGIC_VECTOR ( 31 downto 0 ); 
    q : out STD_LOGIC_VECTOR ( 31 downto 0 ) 
  );
end component;

component sp_fp_mult is
  port (
    rdy : out STD_LOGIC; 
    operation_nd : in STD_LOGIC := 'X'; 
    clk : in STD_LOGIC := 'X'; 
    a : in STD_LOGIC_VECTOR ( 31 downto 0 ); 
    b : in STD_LOGIC_VECTOR ( 31 downto 0 ); 
    result : out STD_LOGIC_VECTOR ( 31 downto 0 ) 
  );
end component;

signal mult1_vld_o,mult1_vld_o_reg: std_logic;

component EAU_pLUT is
  port (
    clk : in STD_LOGIC := 'X'; 
    a : in STD_LOGIC_VECTOR ( 7 downto 0 ); 
    qspo : out STD_LOGIC_VECTOR ( 31 downto 0 ) 
  );
end component;

signal pLUT_data, pLUT_data_reg: std_logic_vector(31 downto 0);
signal pLUTaddr_sftd, pLUTaddr_sftd_fxd: std_logic_Vector(7 downto 0);

component shift_reg_8b_20c is
  port (
    clk : in STD_LOGIC := 'X'; 
    d : in STD_LOGIC_VECTOR ( 7 downto 0 ); 
    q : out STD_LOGIC_VECTOR ( 7 downto 0 ) 
  );
end component;

component CONTROL_AFTER_ITER_FSM is
    Port ( rst : in  STD_LOGIC;
           clk : in  STD_LOGIC;
           vld_in : in  STD_LOGIC;
           sel : out  STD_LOGIC;
           vld_out : out  STD_LOGIC);
end component;

component MUX2to1_32 is
    Port ( sel : in  STD_LOGIC;
           A : in  STD_LOGIC_VECTOR (31 downto 0);
           B : in  STD_LOGIC_VECTOR (31 downto 0);
           RES : out  STD_LOGIC_VECTOR (31 downto 0));
end component;

signal sel, vld_sel: std_logic;
signal MULTA,MULTB: std_logic_vector(31 downto 0);
signal mult_vld_in: std_logic;

component shift_reg_33b_3c is
  port (
    clk : in STD_LOGIC := 'X'; 
    d : in STD_LOGIC_VECTOR ( 32 downto 0 ); 
    q : out STD_LOGIC_VECTOR ( 32 downto 0 ) 
  );
end component;

signal vec33in,vec33out: std_logic_Vector(32 downto 0);

begin

AFTER_ITER_FSM: CONTROL_AFTER_ITER_FSM Port Map
(
 rst => rst,
 clk => clk,
 vld_in => add_vld_o,
 sel => sel,
 vld_out => vld_sel
);

ADDER: sp_fp_add Port Map
(
 rdy => add_vld_o, 
 operation_nd => valid_in, 
 clk => clk,
 a => iter_val,
 b => const_val, 
 result => add_res 
);   

MAINLUT: EAUmLUT Port Map
(
 clk => clk,
 addr => mLUTaddr, 
 data => mLUT_data 
);
  
SHIFTMLUTDATA: shift_reg_32b_9c Port Map 
(
 clk => clk,
 d => mLUT_data, 
 q => mLUT_data_sftd
); 
  
mult_vld_in<=((not sel) and add_vld_o) or (sel and mult1_vld_o_reg);

MULT1: sp_fp_mult Port Map
(
 rdy => mult1_vld_o, 
 operation_nd => mult_vld_in,
 clk => clk, 
 a => MULTA, 
 b => MULTB,
 result => mult1_res 
);

vec33in(32)<=mult1_vld_o ;
vec33in(31 downto 0)<=mult1_res;

INCR_MULT_LAT: shift_reg_33b_3c Port Map
(
 clk => clk, 
 d => vec33in, 
 q => vec33out 
);

SHIFTPOWLUTADDR: shift_reg_8b_20c Port Map
(
 clk => clk, 
 d => pLUTaddr, 
 q => pLUTaddr_sftd
); 

pLUTaddr_sftd_fxd<=pLUTaddr_sftd + "1100111";
  
POWLUT: EAU_pLUT Port Map
(
 clk => clk,
 a => pLUTaddr_sftd_fxd,
 qspo => pLUT_data 
);
  
valid_out<=mult1_vld_o_reg and vld_sel;
value_out<=mult1_res_reg;

mult1_res_reg<=vec33out(31 downto 0);
pLUT_data_reg<=pLUT_data;
mult1_vld_o_reg<=vec33out(32);

MULTPORTA_MUX: MUX2to1_32 Port Map
( sel => sel,
  A => add_res,
  B => mult1_res_reg,
  RES => MULTA
);

MULTPORTB_MUX: MUX2to1_32 Port Map
( sel => sel,
  A => mLUT_data_sftd,
  B => pLUT_data_reg,
  RES => MULTB
);

end Behavioral;

